module TalltyImportExport
  module Attr
    class ExportHeaderItem
      include AttrJson::Model
      attr_json_config(bad_cast: :as_nil, unknown_key: :allow)

      attr_json :key, :string
      attr_json :name, :string
      attr_json :attr_type, :string
      attr_json :format, :string
      attr_json :method, :string
      attr_json :chain, :string, array: true
      attr_json :merge, :boolean
      attr_json :json, :string
      attr_json :select, ActiveModel::Type::Value.new, array: true
      attr_json :source, :boolean
      attr_json :proc, ActiveModel::Type::Value.new
      attr_json :children, self.to_type, array: true

      attr_accessor :depth, :parent_path, :seq

      def calcute_flatten_value _depth, _parent_path
        @depth = _depth
        @parent_path = _parent_path
        if children && children.count > 0
          children.map do |child|
            child.calcute_flatten_value(_depth + 1, [*_parent_path, self])
          end.compact.flatten
        else
          [self]
        end
      end

      def cluster_seqs
        calcute_flatten_value(0, []).map(&:seq)
      end

      def seq
        @seq ||= SecureRandom.uuid
      end

      # def living_alone?
      #   depth == 0 && children.present?
      # end
    end

    class ExportHeader
      include AttrJson::Model
      attr_json_config(bad_cast: :as_nil, unknown_key: :allow)

      attr_json :items, ExportHeaderItem.to_type, array: true

      attr_reader :header_seq_to_axios

      def flatten_value
        items.map do |item|
          item.calcute_flatten_value(0, [])
        end.flatten
      end

      def height
        flatten_value.map(&:depth).max + 1
      end

      def header_lines
        result = []

        height.times.each do |row_index|
          col_index = 0

          next_line = flatten_value.map do |header|
            next_header = [*header.try(:parent_path), header]&.[](row_index)

            @header_seq_to_axios ||= {}


            if next_header
              @header_seq_to_axios[next_header.seq] ||= []
              @header_seq_to_axios[next_header.seq].push([col_index, row_index])
            end

            # 处理最后一列
            if (row_index === height - 1)
              @header_seq_to_axios[header.seq] ||= []
              @header_seq_to_axios[header.seq].push([col_index, row_index])
            end

            col_index += 1

            # 下一级继续递归 或 保持自己以合并
            next_header || header
          end

          result.push(next_line)
        end
        result
      end
    end
  end
end
