module TalltyImportExport
  class Excel
    require 'redis-objects'
    require 'roo'
    require 'roo-xls'

    attr_reader :uid, :cache

    def initialize uid = SecureRandom.hex(8)
      @uid = uid
      @cache = ::Redis::HashKey.new(@uid, expiration: 60 * 60 * 1, marshal: true)
    end

    def load file, **opts
      opts = opts.with_indifferent_access

      file_path = file.is_a?(String) ? file : file.path
      book = ::Roo::Spreadsheet.open(file_path, extension: File.extname(file_path) == '.xls' ? :xls : :xlsx)

      read_title_from_row = opts[:read_title_from_row] || 1
      read_data_from_row = opts[:read_data_from_row] || 2
      titles = book.row(read_title_from_row)
      rows = []

      read_data_from_row.upto(book.last_row) do |i|
        row = book.row(i)[0 .. titles.count]
        out = {}
        titles.each_with_index do |title, index|
          out[title] = row[index]
        end
        rows << out
      end
      cache['uid'] = uid
      cache['rows'] = rows
      cache['titles'] = titles
    end

    def rows
      Rows.new(cache['rows'])
    end

    def titles
      cache['titles']
    end

    def data
      cache.all
    end

    def records_pagination page: 1, per_page: 15
      Pagination.new(rows, page: page, per_page: per_page)
    end

    class Rows < Array
      # excel_hash { key => excel name }
      def each_with_excel_hash excel_hash
        mapping = excel_hash.invert
        each do |row|
          line_info = row.reduce({}) do |out, (k, v)|
            next out unless mapping[k]
            out[mapping[k]] = v
            out
          end
          yield line_info
        end
      end
    end

    class Pagination < Array
      attr_reader :current_page, :total_pages

      def initialize ary, page: ,per_page:
        @raw_ary = ary
        @current_page = page.to_i
        @total_pages = (@raw_ary.count / per_page.to_f).ceil
        parsed_ary = @raw_ary[(page.to_i - 1) * per_page.to_i ... (page.to_i) * per_page.to_i]
        super(parsed_ary)
      end

      def count
        @raw_ary.count
      end
    end
  end
end
