module TalltyImportExport
  class ExportForm < TalltyImportExport::Export
    def initialize *attrs
      super(*attrs)
    end

    class << self
      def export_template_xlsx(form)
        export_xlsx([], {
                      headers: form_transfer_to_headers(form),
                      header_only: true,
                    },)
      end
    end

    def export_workbook workbook, association_records, **options
      # excel导出样式
      alignment = { vertical: :center, horizontal: :center }
      border    = { color: '969696', style: :thin }
      title1 = workbook.styles.add_style(alignment: alignment, border: border, sz: 12, b: true)
      title2 = workbook.styles.add_style(alignment: alignment, border: border, bg_color: '2a5caa', sz: 12, fg_color: 'fffffb')
      title3 = workbook.styles.add_style(alignment: alignment.merge(wrap_text: true), border: border, sz: 10)
      _sheet_name = (respond_to?(:sheet_name) ? sheet_name : nil) || options[:sheet_name]

      header_obj = export_headers_result(**options)

      workbook.add_worksheet(name: _sheet_name) do |sheet|
        index = 0

        if respond_to?(:first_header)
          row_index = Axlsx.col_ref(headers.size - 1)
          sheet.merge_cells("A1:#{row_index}1")
          sheet.add_row [first_header], style: title1, height: 30
          index += 1
        end

        header_obj.header_lines.each do |header_line|
          sheet.add_row(header_line.map(&:name), style: title2, height: 25)
          index += 1
        end

        # 合并相同 header
        header_obj.header_seq_to_axios.values.each do |axios_ary|
          next unless axios_ary.count > 1

          top_right = [axios_ary.map(&:first).min, axios_ary.map(&:last).min]
          bottom_left = [axios_ary.map(&:first).max, axios_ary.map(&:last).max]
          sheet.merge_cells(
            Axlsx.cell_r(top_right.first, top_right.last) + ':' + Axlsx.cell_r(bottom_left.first, bottom_left.last),
          )
        end

        return if options[:header_only]

        value_seq_to_axios = {}

        formats = []
        association_records.each do |association_record|
          records = @each_method.present? ?
            (try_method(association_record, @each_method) || [nil]) :
            [association_record]

          records.each do |record|
            value_living_alone_col_index_to_value_count = {}

            payload = TalltyImportExport::ExportPayload.new(record, header: header_obj) do |payload, header, **opts|
              _data = header[:source] ?
                handle_data(association_record, header, index, **opts) :
                handle_data(payload, header, index, **opts)
            end

            # p '----------------------------------------------------------------'
            # payload.lines.each { |x| p x.map { |x| x.try(:value) || '________' } }

            payload.lines.each_with_index do |line, row_index|
              row = []
              line.each_with_index do |value, col_index|
                value_living_alone_col_index_to_value_count[col_index] ||= 0

                if value.is_a?(TalltyImportExport::ExportPayload::Value)
                  row << value.value
                  value_living_alone_col_index_to_value_count[col_index] += 1
                  value_seq_to_axios[value.seq] = [] unless value_seq_to_axios[value.seq]
                  value_seq_to_axios[value.seq] << [col_index, row_index + index]
                else
                  row << nil
                end
                formats.push(
                  header_obj.flatten_value[col_index].format&.to_sym || (row.last.is_a?(String) ? :string : nil),
                )
              end
              sheet.add_row(row, style: title3, height: @row_height, types: formats)
              formats = []
            end

            # 合并仅有一个值的一列中所有格子
            value_living_alone_col_index_to_value_count.each_pair do |col_index, count|
              if count == 1
                sheet.merge_cells(
                  Axlsx::cell_r(col_index, index) + ':' + Axlsx::cell_r(col_index, index + payload.max_matrix_height - 1)
                )
              end
            end

            index += payload.max_matrix_height
          end
        end

        # 合并相同值
        value_seq_to_axios.values.each do |axios_ary|
          next unless axios_ary.count > 1

          top_right = [axios_ary.map(&:first).min, axios_ary.map(&:last).min]
          bottom_left = [axios_ary.map(&:first).max, axios_ary.map(&:last).max]
          sheet.merge_cells(
            Axlsx.cell_r(top_right.first, top_right.last) + ':' + Axlsx.cell_r(bottom_left.first, bottom_left.last),
          )
        end
      end
    end

    def export_headers_result **options
      @headers = options.deep_symbolize_keys[:headers]
      @headers = super(**options)
      @headers = TalltyImportExport::Attr::ExportHeader.new({ items: @headers })
    end
  end
end
