require 'matrix'

class TalltyImportExport::ExportPayload
  extend ActiveSupport::Autoload

  autoload :Value
  autoload :CellColumn
  autoload :Cell

  attr_reader :value, :context, :cell_columns, :max_matrix_height

  def initialize(val, header:, &value_handler)
    @value = TalltyImportExport::ExportPayload::Value.new(val)
    @headers = header.items # TalltyImportExport::Attr::ExportHeaderItem array
    @flatten_headers = header.flatten_value
    @value_handler = value_handler
    @context = {
      header_seq_to_value_seq_to_value: {},
      parent_cell_seq_to_children_cell_value_array: {},
      flatten_headers: @flatten_headers,
    }
  end

  def lines
    @cell_columns = @headers.map do |header|
      TalltyImportExport::ExportPayload::CellColumn.new(
        header,
        self.value,
        @context,
        &@value_handler
      )
    end

    # 分离的，每个初始 header 为一个矩阵
    matrixes = @cell_columns.map do |cell_column|
      cell_column.tallest_cell_cluster.map(&:grow_to_line)
    end

    # 计算所有矩阵中最高高度，用于填充
    @max_matrix_height = matrixes.map(&:count).max

    # 填充各个矩阵至等高
    detached_matrixes = matrixes.map do |ragged_col|
      col = ragged_col.uniq
      while col.count < @max_matrix_height
        col.push((col[0].try(:count) || 1).times.to_a)
      end
      Matrix[*col]
    end

    # 合并矩阵
    result = detached_matrixes.reduce do |out, matrix|
      Matrix.hstack(out, matrix)
    end.to_a
    # .select do |line|
    #   line.select { |value| TalltyImportExport::ExportPayload::Value === value }.count > 0
    # end

    # result.map { |x| p x.map { |xx| xx.try(:value)}}
    result
  end
end
